==> rulee-ts-0.6.1.tgz: package/lib/esm/dependency-graph/dependency-graph.js <==
import { Logger } from "../util/logger/logger";
import { assertThat } from "../util/assertions/assertions";
import { ListOfPropertiesImpl } from "../properties/list-of-properties-impl";
import { GroupOfPropertiesImpl } from "../properties/group-of-properties-impl";
export class DependencyGraph {
    constructor() {
        this.edgesMap = new Map();
        this.asyncEdgesMap = new Map();
        this.ownerMap = new Map();
    }
    addDependency(from, to, options) {
        var _a, _b;
        Logger.trace(`${from.id} -> ${to.id}: ${Object.keys(options).join(', ')}`);
        let outgoing = this.edgesMap.get(from.id);
        if (!outgoing) {
            outgoing = new Map();
            this.edgesMap.set(from.id, outgoing);
        }
        const existing = outgoing.get(to.id);
        const mergedOptions = existing ? Object.assign(Object.assign({}, existing.options), options) : options;
        outgoing.set(to.id, { from, to, options: mergedOptions });
        const asyncDeps = from.isAsynchronous() ? [from] : this.asyncEdgesMap.get(from.id);
        if (asyncDeps === null || asyncDeps === void 0 ? void 0 : asyncDeps.length) {
            assertThat(() => !this.edgesMap.has(to.id) || to instanceof ListOfPropertiesImpl, () => `DependencyGraph: The property ${to.id} has no dependents, yet`);
            (_b = (_a = this.asyncEdgesMap.get(to.id)) === null || _a === void 0 ? void 0 : _a.push(...asyncDeps)) !== null && _b !== void 0 ? _b : this.asyncEdgesMap.set(to.id, asyncDeps);
        }
    }
    addDependencies(from, to, options) {
        from.forEach((prop) => this.addDependency(prop, to, options));
    }
    addOwnerDependency(owner, ownedProperty, withValueDependency) {
        if (withValueDependency !== false) {
            this.addDependency(ownedProperty, owner, { value: true });
        }
        const owned = this.ownerMap.get(owner.id);
        if (owned) {
            owned.push(ownedProperty.id);
        }
        else {
            this.ownerMap.set(owner.id, [ownedProperty.id]);
        }
    }
    traverseDepthFirst(start, apply, filter) {
        const outgoing = this.edgesMap.get(start);
        if (outgoing) {
            outgoing.forEach((dependency, to) => {
                if (!filter || filter(dependency)) {
                    apply(dependency.to, dependency);
                    this.traverseDepthFirst(to, apply, filter);
                }
            });
        }
    }
    getAsyncDependencies(propertyId) {
        return this.asyncEdgesMap.get(propertyId);
    }
    findCyclicDependencies() {
        const visited = new Set();
        for (const propId in this.edgesMap.keys()) {
            const cyclicPath = this.findCyclicDependenciesRecursive(propId, [], visited);
            if (cyclicPath) {
                return cyclicPath;
            }
        }
        return null;
    }
    findCyclicDependenciesRecursive(current, path, visited) {
        if (visited.has(current)) {
            const currentIndex = path.indexOf(current);
            if (currentIndex >= 0) {
                const cylicPath = path.slice(currentIndex);
                cylicPath.push(current);
                return cylicPath;
            }
        }
        else {
            visited.add(current);
            path.push(current);
            const outgoing = this.edgesMap.get(current);
            outgoing === null || outgoing === void 0 ? void 0 : outgoing.forEach(dep => this.findCyclicDependenciesRecursive(dep.to.id, path, visited));
        }
        return null;
    }
    createVisJsData(allProperties) {
        const nodes = [];
        const edges = [];
        const propertyIdVisJsNodeMap = this.createVisJsNodes(allProperties, nodes);
        this.createVisJsEdges(propertyIdVisJsNodeMap, edges);
        const options = {
            nodes: {
                shape: "dot",
                size: 30,
                font: {
                    size: 15
                },
                borderWidth: 2,
                shadow: true
            },
            groups: {
                'scalar': {
                    color: { background: "#37278C", border: "black" }
                },
                'scalar-async': {
                    color: { background: "#4F419B", border: "black" }
                },
                'group': {
                    color: { background: "#40AC1E", border: "black" },
                    shape: "hexagon"
                },
                'group-async': {
                    color: { background: "#5EBE3F", border: "black" },
                    shape: "hexagon"
                },
                'list': {
                    color: { background: "#C12237", border: "black" },
                    shape: "square"
                },
                'list-async': {
                    color: { background: "#D5475B", border: "black" },
                    shape: "square"
                }
            },
            edges: {
                width: 2,
                shadow: true
            }
        };
        return { nodes, edges, options };
    }
    createVisJsNodes(allProperties, nodes) {
        const propertyIdVisJsNodeMap = new Map();
        allProperties.forEach((property) => {
            const node = {
                id: 1 + propertyIdVisJsNodeMap.size,
                label: property.id,
                group: this.propertyToVisNodeGroup(property)
            };
            nodes.push(node);
            propertyIdVisJsNodeMap.set(property.id, node);
        });
        return propertyIdVisJsNodeMap;
    }
    propertyToVisNodeGroup(property) {
        const isAsync = property.isAsynchronous();
        if (property instanceof ListOfPropertiesImpl) {
            return isAsync ? 'list-async' : 'list';
        }
        if (property instanceof GroupOfPropertiesImpl) {
            return isAsync ? 'group-async' : 'group';
        }
        return isAsync ? 'scalar-async' : 'scalar';
    }
    createVisJsEdges(map, edges) {
        this.edgesMap.forEach(outgoing => {
            outgoing.forEach(dependency => {
                const edge = {
                    from: map.get(dependency.from.id).id,
                    to: map.get(dependency.to.id).id,
                    dashes: !dependency.options.value,
                    title: Object.entries(dependency.options).filter(keyVal => !!keyVal[1]).map(keyVal => !keyVal[0]).join(', '),
                    arrows: {
                        to: {
                            enabled: true,
                            type: 'triangle'
                        }
                    }
                };
                edges.push(edge);
            });
        });
        this.ownerMap.forEach((toList, from) => {
            toList.forEach(to => {
                const edge = {
                    from: map.get(from).id,
                    to: map.get(to).id,
                    color: "rgb(110, 110, 110)",
                    arrows: {
                        from: {
                            enabled: true,
                            type: 'diamond'
                        }
                    }
                };
                edges.push(edge);
            });
        });
    }
}

==> rulee-ts-0.8.0.tgz: package/lib/esm/dependency-graph/dependency-graph.js <==
import { Logger } from "../util/logger/logger";
import { assertThat } from "../util/assertions/assertions";
import { ListOfPropertiesImpl } from "../properties/list-of-properties-impl";
import { GroupOfPropertiesImpl } from "../properties/group-of-properties-impl";
export class DependencyGraph {
    constructor() {
        this.edgesMap = new Map();
        this.asyncEdgesMap = new Map();
        this.ownerMap = new Map();
    }
    addDependency(from, to, options) {
        var _a, _b;
        Logger.trace(`${from.id} -> ${to.id}: ${Object.keys(options).join(', ')}`);
        let outgoing = this.edgesMap.get(from.id);
        if (!outgoing) {
            outgoing = new Map();
            this.edgesMap.set(from.id, outgoing);
        }
        const existing = outgoing.get(to.id);
        const mergedOptions = existing ? Object.assign(Object.assign({}, existing.options), options) : options;
        outgoing.set(to.id, { from, to, options: mergedOptions });
        const asyncDeps = from.isAsynchronous() ? [from] : this.asyncEdgesMap.get(from.id);
        if (asyncDeps === null || asyncDeps === void 0 ? void 0 : asyncDeps.length) {
            assertThat(() => !this.edgesMap.has(to.id) || to instanceof ListOfPropertiesImpl, () => `DependencyGraph: The property ${to.id} has no dependents, yet`);
            (_b = (_a = this.asyncEdgesMap.get(to.id)) === null || _a === void 0 ? void 0 : _a.push(...asyncDeps)) !== null && _b !== void 0 ? _b : this.asyncEdgesMap.set(to.id, asyncDeps);
        }
    }
    addDependencies(from, to, options) {
        from.forEach((prop) => this.addDependency(prop, to, options));
    }
    addOwnerDependency(owner, ownedProperty, withValueDependency) {
        if (withValueDependency !== false) {
            this.addDependency(ownedProperty, owner, { value: true });
        }
        const owned = this.ownerMap.get(owner.id);
        if (owned) {
            owned.push(ownedProperty.id);
        }
        else {
            this.ownerMap.set(owner.id, [ownedProperty.id]);
        }
    }
    traverseDepthFirst(start, apply, filter, preventCycles) {
        let visited;
        const outgoing = this.edgesMap.get(start);
        if (outgoing) {
            outgoing.forEach((dependency, to) => {
                if (!filter || filter(dependency)) {
                    if (preventCycles) {
                        if (!visited) {
                            visited = new Set();
                        }
                        if (!visited.has(dependency.to.id)) {
                            apply(dependency.to, dependency);
                            visited.add(dependency.to.id);
                            this.traverseDepthFirst(to, apply, filter);
                        }
                    }
                    else {
                        apply(dependency.to, dependency);
                        this.traverseDepthFirst(to, apply, filter);
                    }
                }
            });
        }
    }
    getAsyncDependencies(propertyId) {
        return this.asyncEdgesMap.get(propertyId);
    }
    findCyclicDependencies() {
        const visited = new Set();
        for (const propId in this.edgesMap.keys()) {
            const cyclicPath = this.findCyclicDependenciesRecursive(propId, [], visited);
            if (cyclicPath) {
                return cyclicPath;
            }
        }
        return null;
    }
    findCyclicDependenciesRecursive(current, path, visited) {
        if (visited.has(current)) {
            const currentIndex = path.indexOf(current);
            if (currentIndex >= 0) {
                const cylicPath = path.slice(currentIndex);
                cylicPath.push(current);
                return cylicPath;
            }
        }
        else {
            visited.add(current);
            path.push(current);
            const outgoing = this.edgesMap.get(current);
            outgoing === null || outgoing === void 0 ? void 0 : outgoing.forEach(dep => this.findCyclicDependenciesRecursive(dep.to.id, path, visited));
        }
        return null;
    }
    createVisJsData(allProperties) {
        const nodes = [];
        const edges = [];
        const propertyIdVisJsNodeMap = this.createVisJsNodes(allProperties, nodes);
        this.createVisJsEdges(propertyIdVisJsNodeMap, edges);
        const options = {
            nodes: {
                shape: "dot",
                size: 30,
                font: {
                    size: 15
                },
                borderWidth: 2,
                shadow: true
            },
            groups: {
                'scalar': {
                    color: { background: "#37278C", border: "black" }
                },
                'scalar-async': {
                    color: { background: "#4F419B", border: "black" }
                },
                'group': {
                    color: { background: "#40AC1E", border: "black" },
                    shape: "hexagon"
                },
                'group-async': {
                    color: { background: "#5EBE3F", border: "black" },
                    shape: "hexagon"
                },
                'list': {
                    color: { background: "#C12237", border: "black" },
                    shape: "square"
                },
                'list-async': {
                    color: { background: "#D5475B", border: "black" },
                    shape: "square"
                }
            },
            edges: {
                width: 2,
                shadow: true
            }
        };
        return { nodes, edges, options };
    }
    createVisJsNodes(allProperties, nodes) {
        const propertyIdVisJsNodeMap = new Map();
        allProperties.forEach((property) => {
            const node = {
                id: 1 + propertyIdVisJsNodeMap.size,
                label: property.id,
                group: this.propertyToVisNodeGroup(property)
            };
            nodes.push(node);
            propertyIdVisJsNodeMap.set(property.id, node);
        });
        return propertyIdVisJsNodeMap;
    }
    propertyToVisNodeGroup(property) {
        const isAsync = property.isAsynchronous();
        if (property instanceof ListOfPropertiesImpl) {
            return isAsync ? 'list-async' : 'list';
        }
        if (property instanceof GroupOfPropertiesImpl) {
            return isAsync ? 'group-async' : 'group';
        }
        return isAsync ? 'scalar-async' : 'scalar';
    }
    createVisJsEdges(map, edges) {
        this.edgesMap.forEach(outgoing => {
            outgoing.forEach(dependency => {
                const edge = {
                    from: map.get(dependency.from.id).id,
                    to: map.get(dependency.to.id).id,
                    dashes: !dependency.options.value,
                    title: Object.entries(dependency.options).filter(keyVal => !!keyVal[1]).map(keyVal => !keyVal[0]).join(', '),
                    arrows: {
                        to: {
                            enabled: true,
                            type: 'triangle'
                        }
                    }
                };
                edges.push(edge);
            });
        });
        this.ownerMap.forEach((toList, from) => {
            toList.forEach(to => {
                const edge = {
                    from: map.get(from).id,
                    to: map.get(to).id,
                    color: "rgb(110, 110, 110)",
                    arrows: {
                        from: {
                            enabled: true,
                            type: 'diamond'
                        }
                    }
                };
                edges.push(edge);
            });
        });
    }
}
